@@ -165,6 +165,16 @@ def retry(value):
         if len(fields) < 2:
             raise ValueError("needs at least 2 values")
         pattern, count, timeout, delay, backoff = [*fields, *["", None, None, 0, 1][len(fields):]]
+        if not pattern:
+            pattern = ""
+        if not count:
+            count = None
+        if not timeout:
+            timeout = None
+        if not delay:
+            delay = 0
+        if not backoff:
+            backoff = 1
         option = Retry(count=count, timeout=timeout, delay=delay, backoff=backoff, jitter=jitter, pattern=pattern)
     except Exception as e:
         raise ArgumentTypeError(f"'{value}' is invalid")
@@ -185,10 +195,12 @@ def tags_filter(value):
     return option
 
 def onoff(value):
-    if value in ["yes", "1", "on"]:
+    if value.lower() in ["yes", "1", "on"]:
         return True
-    elif value in ["no", "0", "off"]:
+    elif value.lower() in ["no", "0", "off"]:
         return False
     elif value == NoneValue:
         return NoneValue
     raise ArgumentTypeError(f"'{value}' is invalid")
+
+onoff.metavar = str(set(["yes", "1", "on", "no", "0", "off"])).replace(" ","").replace("'","")
\ No newline at end of file
