@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # to the end flag
+import uuid
 import atexit
 import weakref
 import itertools
@@ -22,6 +23,7 @@ import concurrent.futures.thread as _base
 from .future import Future 
 from .. import _get_parallel_context
 from ..asyncio import is_running_in_event_loop, asyncio
+from .. import current, join as parallel_join
 
 _tasks_queues = weakref.WeakKeyDictionary()
 _shutdown = False
@@ -113,7 +115,7 @@ class AsyncPoolExecutor(_base._base.Executor):
     _counter = itertools.count().__next__
 
     def __init__(self, max_workers=1024, task_name_prefix="",
-            loop=None, _check_max_workers=True):
+            loop=None, _check_max_workers=True, join_on_shutdown=True):
         if _check_max_workers and int(max_workers) <= 0:
             raise ValueError("max_workers must be greater than 0")
         self._open = False
@@ -127,6 +129,8 @@ class AsyncPoolExecutor(_base._base.Executor):
         self._task_name_prefix = (task_name_prefix or
             ("AsyncPoolExecutor-%d" % self._counter()))
         self._async_loop_thread = None
+        self._uid = str(uuid.uuid1())
+        self._join_on_shutdown = join_on_shutdown
 
     @property
     def open(self):
@@ -157,6 +161,8 @@ class AsyncPoolExecutor(_base._base.Executor):
                 raise RuntimeError("cannot schedule new futures after interpreter shutdown")
 
             future = Future()
+            future._executor_uid = self._uid
+
             ctx = _get_parallel_context()
             args = fn, *args
             work_item = _AsyncWorkItem(future, ctx.run, args, kwargs)
@@ -173,9 +179,6 @@ class AsyncPoolExecutor(_base._base.Executor):
                 raise RuntimeError("deadlock detected")
             asyncio.run_coroutine_threadsafe(work_item.run(), loop=self._loop).result()
 
-        if is_running_in_event_loop():
-            return asyncio.wrap_future(future)
-
         return future
 
     def _adjust_task_count(self):
@@ -199,11 +202,7 @@ class AsyncPoolExecutor(_base._base.Executor):
 
         return False
 
-    def __exit__(self, exc_type, exc_value, exc_tb):
-        self.shutdown(wait=True)
-        return False
-
-    def shutdown(self, wait=True):
+    def shutdown(self, wait=True, test=None):
         with self._shutdown_lock:
             if self._shutdown:
                 return
@@ -214,33 +213,41 @@ class AsyncPoolExecutor(_base._base.Executor):
                 self._work_queue.put(None), loop=self._loop).result()
 
         if wait:
-            exc = None
-            for task in self._tasks:
-                try:
-                    task.result()
-                except BaseException as e:
-                    exc = e
+            if test is None:
+                test = current()
             try:
-                if exc is not None:
-                    raise exc
+                if test:
+                    if self._join_on_shutdown:
+                        parallel_join(no_async=True, test=test, filter=lambda future: hasattr(future, "_executor_uid") and future._executor_uid == self._uid)
             finally:
-                asyncio.run_coroutine_threadsafe(
-                        _loop_send_stop_event(self._loop_stop_event), loop=self._loop
-                    ).result()
-                if self._async_loop_thread is not None:
-                    self._async_loop_thread.join()
+                exc = None
+                for task in self._tasks:
+                    try:
+                        task.result()
+                    except BaseException as e:
+                        exc = e
+                try:
+                    if exc is not None:
+                        raise exc
+                finally:
+                    asyncio.run_coroutine_threadsafe(
+                            _loop_send_stop_event(self._loop_stop_event), loop=self._loop
+                        ).result()
+                    if self._async_loop_thread is not None:
+                        self._async_loop_thread.join()
 
 
 class SharedAsyncPoolExecutor(AsyncPoolExecutor):
     """Shared async pool executor.
     """
-    def __init__(self, max_workers, task_name_prefix=""):
-        self.initargs = (max_workers, task_name_prefix) 
+    def __init__(self, max_workers, task_name_prefix="", join_on_shutdown=True):
+        self.initargs = (max_workers, task_name_prefix, join_on_shutdown) 
 
         if int(max_workers) < 0:
             raise ValueError("max_workers must be positive or 0")
         super(SharedAsyncPoolExecutor, self).__init__(
-            max_workers=max_workers-1, task_name_prefix=task_name_prefix, _check_max_workers=False)
+            max_workers=max_workers-1, task_name_prefix=task_name_prefix,
+            _check_max_workers=False, join_on_shutdown=join_on_shutdown)
 
     def submit(self, fn, args=None, kwargs=None, block=False):
         return super(SharedAsyncPoolExecutor, self).submit(fn=fn, args=args, kwargs=kwargs, block=block)
