@@ -20,7 +20,7 @@ import atexit
 import inspect
 import traceback
 import threading
-
+import concurrent.futures
 import testflows.settings as settings
 
 from collections import namedtuple
@@ -30,11 +30,11 @@ from testflows._core.contrib import cloudpickle
 from testflows._core.contrib.aiomsg import Socket
 
 from .asyncio import asyncio, is_running_in_event_loop, CancelledError
-
+from .executor.thread import SharedThreadPoolExecutor
 
 Address = namedtuple("address", "hostname port", defaults=(None,))
 
-TimeoutError = asyncio.exceptions.TimeoutError
+TimeoutError = (asyncio.exceptions.TimeoutError, concurrent.futures._base.TimeoutError)
 
 
 class ServiceError(Exception):
@@ -92,6 +92,7 @@ class Service:
         self.serve_tasks = []
         self.reply_events = {}
         self.objects = {}
+        self.executor = SharedThreadPoolExecutor(sys.maxsize, join_on_shutdown=False)
         self.open = False
         self.lock = asyncio.Lock(loop=self.loop)
 
@@ -145,7 +146,7 @@ class Service:
             _id = id(obj)
 
             if _id in self.objects:
-                del self.objects[id(obj)]
+                del self.objects[_id]
         
         if loop is None:
             return asyncio.run_coroutine_threadsafe(_async_unregister(), loop=self.loop).result()
@@ -208,13 +209,13 @@ class Service:
                 self.reply_events[rid] = event
                 return event
 
-        async with self.lock:
-            if not self.open:
-                raise ServiceNotRunningError("service is not running")
+        if not self.open:
+            raise ServiceNotRunningError("service is not running")
             
-            if address == self.address:
-                return local_send
+        if address == self.address:
+            return local_send
 
+        async with self.lock:
             if not address in self.connections:
                 event = asyncio.Event()
                 await self.out_socket.connect(address.hostname, address.port, event=event)
@@ -236,6 +237,7 @@ class Service:
         """Async context manager enter.
         """
         async with self.lock:
+            self.executor.__enter__()
             await self.in_socket.bind(hostname=self.address.hostname, port=self.address.port)
             self.address = Address(*self.in_socket.bind_address)
             self.loop.create_task(self._serve_forever())
@@ -261,6 +263,7 @@ class Service:
 
                 await self.out_socket.close()
                 await self.in_socket.close()
+                self.executor.__exit__(None, None, None)
             finally:
                 self.open = False
 
@@ -315,7 +318,7 @@ class Service:
             elif fn == "__decref__":
                 r = self.__decref__(oid, obj_item)
             else:
-                r = await self.loop.run_in_executor(None, r)
+                r = await self.loop.run_in_executor(self.executor, r)
 
             if asyncio.iscoroutine(r):
                 r = await r
@@ -336,7 +339,6 @@ class Service:
             # process request
             oid, fn, args, kwargs = msg_body
             msg_type, r = await self._exec(oid, fn, args, kwargs)
-
             try:
                 await self.in_socket.send_pickle((msg_type, rid, r), identity=identity)
             except TypeError as e:
@@ -390,11 +392,12 @@ def process_service(**kwargs):
                     asyncio.set_event_loop(loop)
                     try:
                         loop.run_forever()
-                    finally:                       
+                    finally:
                         tasks = [task for task in asyncio.all_tasks(loop=loop)]
                         for task in tasks:
                             task.cancel()
                         loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
+                        loop.run_until_complete(loop.shutdown_asyncgens())
                         loop.close()
 
                 thread = threading.Thread(target=run_event_loop, daemon=True)
@@ -419,7 +422,7 @@ def _stop_process_service():
     """Stop global process service.
     """
     global _process_service
- 
+
     with _process_service_lock:
         if _process_service is not None:
             _process_service.__exit__(None, None, None)
@@ -435,7 +438,7 @@ class BaseServiceObject:
     _exposed = None
     _typename = None
 
-    def __init__(self, oid, address, _incref=True) -> None:
+    def __init__(self, oid, address, _incref=True):
         """Initialize service object.
         """
         self.oid = oid
@@ -464,7 +467,10 @@ class BaseServiceObject:
         if is_running_in_event_loop():
             if _process_service: 
                 if _process_service.loop is asyncio.get_running_loop():
-                    _process_service.__incref__(oid)
+                    if  _process_service.address == address:
+                        _process_service.__incref__(oid)
+                    else:
+                        _process_service.loop.create_task(BaseServiceObject.__async_proxy_call__(oid, address, "__incref__"))
                     return
 
         BaseServiceObject.__proxy_call__(oid, address, "__incref__")
@@ -481,9 +487,18 @@ class BaseServiceObject:
 
         try:
             if is_running_in_event_loop():
-                if _process_service: 
-                    if _process_service.loop is asyncio.get_running_loop():  
-                        _process_service.__decref__(oid)
+                async def no_errors(aws):
+                    try:
+                        await aws
+                    except (ServiceNotRunningError, ServiceObjectNotFoundError, CancelledError):
+                        pass
+
+                if _process_service:
+                    if _process_service.loop is asyncio.get_running_loop():
+                        if _process_service.address == address:  
+                            _process_service.__decref__(oid)
+                        else:
+                            _process_service.loop.create_task(no_errors(BaseServiceObject.__async_proxy_call__(oid, address, "__decref__")))
                         return
 
             BaseServiceObject.__proxy_call__(
@@ -521,7 +536,7 @@ class BaseServiceObject:
                 return await c
 
             send = await wrap(_process_service._connect(address))
-            response = await wrap(send(uuid.uuid1().bytes, oid, fn, args, kwargs, resp=True, timeout=timeout))
+            response = await wrap(send(uuid.uuid1().hex, oid, fn, args, kwargs, resp=True, timeout=timeout))
 
             await asyncio.wait_for(wrap(response.wait()), timeout=timeout)
             reply_type, reply_body = response.message
@@ -542,7 +557,11 @@ class BaseServiceObject:
         """Synchronously execute function call on the remote service.
         """
         try:
-            return asyncio.run_coroutine_threadsafe(BaseServiceObject.__async_proxy_call__(oid, address, fn, args, kwargs, timeout=timeout), loop=_process_service.loop).result()
+            c = BaseServiceObject.__async_proxy_call__(oid, address, fn, args, kwargs, timeout=timeout)
+            try:
+                return asyncio.run_coroutine_threadsafe(c, loop=_process_service.loop).result()
+            finally:
+                c.close()
         except AttributeError:
             if _process_service is None:
                 raise ServiceNotRunningError("service has not been started")
