@@ -39,7 +39,7 @@ show_skipped = False
 #: show retries
 show_retries = False
 #: show trimmed result messages
-trim = False
+trim_results = False
 #: randomize order of loaded tests
 random_order = False
 #: global thread pool
@@ -50,3 +50,5 @@ global_async_pool = None
 global_process_pool = None
 #: service timeout
 service_timeout = 0.1
+#: secrets registry
+secrets_registry = None
